package WebService::MyGengo;

our $VERSION = '0.011';
use v5.10.0; # todo We use the // operator

=head1 NAME

WebService::MyGengo - A library for interacting with the myGengo API

=head1 DESCRIPTION

L<WebService::MyGengo> is a library for accessing the MyGengo
(L<http://mygengo.com>) API.

As of this writing, all interaction with the API is handled via the
L<WebService::MyGengo::Client> class, in which you will find more enlightening
documentation.

=head1 SEE ALSO

This module on GitHub: L<https://github.com/nheinric/WebService--MyGengo>

=head1 AUTHOR

Nathaniel Heinrichs

=head1 LICENSE

Copyright (c) 2011, Nathaniel Heinrichs <nheinric-at-cpan.org>.
All rights reserved.

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
